"""
fastclash: Linux Clash 代理工具快速安装器

通过 PyPI 提供便捷的 Clash 安装渠道。
包含完整的 Shell 脚本和资源文件，通过 fastclash init 快速部署。

核心功能：
- fastclash init: 部署脚本到系统并初始化环境
- 所有业务逻辑由高效的 Shell 脚本处理

项目地址: https://github.com/whillhill/fastclash
"""

__version__ = "1.2.12"
__author__ = "whillhill"
__email__ = "ooooofish@126.com"
__license__ = "MIT"
__url__ = "https://github.com/whillhill/fastclash"

__all__ = [
    "__version__",
    "__author__",
    "__email__",
    "__license__",
    "__url__",
]
